#include <kpixmapeffect.h> 

bool LiquidStyle::eventFilter( QObject *obj, QEvent *ev )
{
    switch (ev->type()){
        case QEvent::None:
            case QEvent::Timer:{
                // first check if this is one of our fading pushbuttons - return elsewise
                if (!::qt_cast<QPushButton*>(obj))
                    return FALSE;
                ButtonFadeInfo *bfi_ = bfi[((QPushButton*)obj)->winId()];
                if (!bfi_ || ((QTimerEvent*)ev)->timerId() != bfi_->timerId)
                    return FALSE;
            // ok, this is one of our fading buttons asking for repaint
            ((QPushButton*)obj)->repaint(FALSE);
            int index = bfi_->fadeIn?bfi_->index + 1:bfi_->index - 1;
            // handle border cases
            if (index < 0) // fade-out run has finished, either stop fading stuff, or start fade-in
            {
                bfi_->fadeIn=TRUE;
                if (((QPushButton*)obj)->isDefault())
                {
                    bfi_->index=0;
                }
                else
                {
                    obj->killTimer(bfi_->timerId);
//                     qWarning("%u: stopped (%d)", ((QPushButton*)obj)->winId(), bfi_->timerId);
                    bfi_->timerId=0;
                    ((QPushButton*)obj)->repaint(FALSE); // must be to clean out button
                }
            }
            else if (index > 9) // fade-in run has finished, either stop fading stuff, or start fade-out
            {
                if (((QPushButton*)obj)->isDefault())
                {
                    bfi_->fadeIn=FALSE;
                    bfi_->index=9;
                }
                else
                {
                    obj->killTimer(bfi_->timerId);
//                     qWarning("%u: stopped (%d)", ((QPushButton*)obj)->winId(), bfi_->timerId);
                    bfi_->timerId=0;
                }
            }
            else
                bfi_->index = index;
            }
        case QEvent::MouseMove:
//             qWarning("MouseMove");
            if (::qt_cast<QTabBar*>(obj)){
                QTabBar *tb = (QTabBar*)obj;
                QRect tmpRect;
                bool done = false;
                QMouseEvent *me = ( QMouseEvent*)ev;
                int oldTab = hoveredTabIndex;
                if (hoveredTabIndex == -1){
                    for (int i = 0; i < tb->count(); i++){
                        if (tb->tabAt(i)){
                            tmpRect = tb->tabAt(i)->rect();
                            if ( tmpRect.x() < me->x() && me->x() < tmpRect.right()){
                                hoveredTabIndex = i;
                                break;
                            }
                        }
                    }
                }
                else{
                    if ((oldTab > 0) && tb->tabAt(oldTab-1)){
                        tmpRect = tb->tabAt(oldTab-1)->rect();
                        if ((tmpRect.x() < me->x()) && (me->x() < tmpRect.right())){
                            hoveredTabIndex = oldTab-1;
                            done = true;
                        }
                    }
                    if (!done && (oldTab < tb->count()-1) && tb->tabAt(oldTab+1)){
                        tmpRect = tb->tabAt(oldTab+1)->rect();
                        if ((tmpRect.x() < me->x()) && (me->x() < tmpRect.right())){
                            hoveredTabIndex = oldTab+1;
                        }
                    }
                }
                if ( oldTab != hoveredTabIndex ) {
                    // reset old tab
                    if ( oldTab != -1 ) {
                        if (tb->tabAt(oldTab)) tb->repaint( tb->tabAt(oldTab)->rect(), false );
                    }
                    if ( hoveredTabIndex != -1 ) {
                        tb->repaint( tmpRect, false );
                    }
                }
                return FALSE;
            }
            else if ( ::qt_cast<QHeader*>(obj) ) {
                QHeader *hw = (QHeader*)obj;
                QMouseEvent * me = ( QMouseEvent * ) ev;
                int oldHeader = headerHoverID;
                headerHoverID = hw->sectionAt( me->x() + hw->offset());
                if ( oldHeader != headerHoverID ) {
                    // reset old header
                    if ( oldHeader != -1 ) {
                        hw->repaint( hw->sectionPos( oldHeader )-hw->offset(), 0, hw->sectionSize( oldHeader ), hw->height(), false );
                    }
                    if ( headerHoverID != -1 ) {
                            hw->repaint( hw->sectionPos( headerHoverID )-hw->offset(), 0, hw->sectionSize( headerHoverID ), hw->height(), false );
                    }
                }
                return FALSE;
            }
            return FALSE;
        case QEvent::Paint:
//             qWarning("Paint");
            if ( ::qt_cast<QToolButton*>(obj) ) {
                QToolButton * btn = ( QToolButton * ) obj;
                if (!obj->inherits( "TaskContainer" )){
                    if (btn->backgroundOrigin() != QWidget::WindowOrigin)
                        btn->setBackgroundOrigin( QWidget::WindowOrigin );
                }
                else{
                    isTaskContainer = true;
                    if ( currentTaskContainer == btn )
                        taskContainerHover = true;
                    else
                        taskContainerHover = false;
                }
                return FALSE;
            }
            else if (::qt_cast<QMenuBar*>(obj)){
//                 if (KStyle::eventFilter( obj, ev ))
//                     return TRUE;
                QWidget *w = (QWidget *)obj;
                if(w->backgroundMode() != Qt::PaletteBackground){
                    w->setBackgroundMode(PaletteBackground);
                    w->setBackgroundOrigin(QWidget::WindowOrigin);
                }
                return FALSE;
            }
            else if ( ::qt_cast<QHeader*>(obj) ) {
                QHeader *hw = (QHeader*)obj;
                if (hw->parentWidget() && ::qt_cast<QListView*>(hw->parentWidget()))
                    headerSortID = ((QListView*)hw->parentWidget())->sortColumn();
                return FALSE;
            }
            else if ( obj->inherits( "QDockWindow" ) ) {
                if (KStyle::eventFilter( obj, ev ))
                    return TRUE;
                QWidget * w = ( QWidget * ) obj;
                if ( w->backgroundMode() != Qt::PaletteBackground ) {
                    w->setBackgroundMode( Qt::PaletteBackground );
                    w->setBackgroundOrigin( QWidget::WindowOrigin );
                }
                return FALSE;
            }
            else if ( obj->inherits( "QRadioButton" ) || obj->inherits( "QCheckBox" ) ) {
                QButton * btn = ( QButton * ) obj;
                bool isStatusChild = btn->parent() && ( btn->parent() ->inherits( "QStatusBar" ) || btn->parent() ->inherits( "KonqFrameStatusBar" ) );
                bool isRadio = obj->inherits( "QRadioButton" );
                btn->erase();
                QPainter p;
                p.begin( btn );
                QFontMetrics fm = btn->fontMetrics();
                QSize lsz = fm.size( ShowPrefix, btn->text() );
                QSize sz = isRadio ?
                QSize( pixelMetric( PM_ExclusiveIndicatorWidth ), pixelMetric( PM_ExclusiveIndicatorHeight ) ) :
                QSize( pixelMetric( PM_IndicatorWidth ), pixelMetric( PM_IndicatorHeight ) );
                int x = 0;
                int y = isStatusChild ? 0 : ( btn->height() - lsz.height() + fm.height() - sz.height() ) / 2;
                SFlags flags = Style_Default;
                if ( btn->isEnabled() )
                    flags |= Style_Enabled;
                if ( btn->hasFocus() )
                    flags |= Style_HasFocus;
                if ( btn->isDown() )
                    flags |= Style_Down;
                if ( highlightWidget == btn )
                    flags |= Style_MouseOver;
                if ( btn->state() == QButton::On )
                    flags |= Style_On;
                else if ( btn->state() == QButton::Off )
                    flags |= Style_Off;
                if ( isRadio )
                    drawControl( CE_RadioButton, &p, btn, QRect( x, y, sz.width(), sz.height() ), btn->colorGroup(), flags );
                else
                    drawControl( CE_CheckBox, &p, btn, QRect( x, y, sz.width(), sz.height() ), btn->colorGroup(), flags );
                x = sz.width() + 6;
                y = 0;
                if ( btn->hasFocus() ) {
                    QColor tmp( btn->colorGroup().background().dark( 130 ) );
                    drawItem( &p, QRect( sz.width() + 6 + 2, 1, btn->width() - ( sz.width() + 6 + 1 ), btn->height() ), AlignLeft | AlignVCenter | ShowPrefix, btn->colorGroup(), btn->isEnabled(), btn->pixmap(), btn->text(), -1, &tmp );
                    drawItem( &p, QRect( sz.width() + 6 + 3, 2, btn->width() - ( sz.width() + 6 + 1 ), btn->height() ), AlignLeft | AlignVCenter | ShowPrefix, btn->colorGroup(), btn->isEnabled(), btn->pixmap(), btn->text(), -1, &tmp );
                }
                drawItem( &p, QRect( sz.width() + 6 + 1, 0, btn->width() - ( sz.width() + 6 + 1 ), btn->height() ), AlignLeft | AlignVCenter | ShowPrefix, btn->colorGroup(), btn->isEnabled(), btn->pixmap(), btn->text() );
                p.end();
                return TRUE;
            }
            else if (obj->inherits("KColorGrid") && !(((QWidget*)obj)->width() && ((QWidget*)obj)->height())){
                return TRUE;
            }
            return FALSE;
        case QEvent::KeyPress:
//             qWarning("KeyPress");
//             if (::qt_cast<QMenuBar*>(obj)){
//                 optionHandler->mouseOverMenu = true;
//             }
            return FALSE;
        case QEvent::KeyRelease:
            return FALSE;
        case QEvent::Wheel:
            if ( ::qt_cast<QScrollView*>(obj) ) {
                optionHandler->setScrollerActive();
                return FALSE;
            }
            return FALSE;
        case QEvent::Enter:
//             qWarning("Enter");
            if ( ::qt_cast<QToolButton*>(obj) ) {
                bool isTask = obj->inherits( "TaskContainer" );
                QToolButton * btn = ( QToolButton * ) obj;
                if (isTask){
                    currentTaskContainer = btn;
                    btn->repaint( false );
                    return FALSE;
                }
                else if ( btn->isEnabled() ) {
                    highlightWidget = btn;
                    btn->repaint( false );
                    return FALSE;
                }
            }
            else if ( ::qt_cast<QPushButton*>(obj) || ::qt_cast<QComboBox*>(obj) || obj->inherits( "QSplitterHandle" ) || obj->inherits( "AppletHandleDrag" ) ) {
                QWidget * btn = ( QWidget * ) obj;
                if ( btn->isEnabled() ) {
                    highlightWidget = btn;
                    btn->repaint( false );
                    if (::qt_cast<QPushButton*>(obj))
                        fadeIn((QPushButton*)obj);
                }
                return FALSE;
            }
            else if (::qt_cast<KToolBar*>(obj)){
                QObjectList *handlerList = obj->queryList( "QDockWindowHandle" );
                for (uint i= 0; i < handlerList->count(); i++)
                    ((QWidget*)(handlerList->at(i)))->repaint(false);
                return FALSE;
            }
//             else if (::qt_cast<QMenuBar*>(obj)){
//                 optionHandler->mouseOverMenu = true;
//                 return FALSE;
//             }
            /*else*/ if ( ::qt_cast<QScrollBar*>(obj) ) {
                QScrollBar * sb = ( QScrollBar * ) obj;
                if ( !sb->draggingSlider() ) {
                    if ( sb->isEnabled() ) {
                        activeScroller = sb;
                        highlightWidget = sb;
                        sb->repaint( false );
                    }
                }
                return FALSE;
            }
            else if ( ::qt_cast<QScrollView*>(obj) ) {
                QScrollView *sv = ( QScrollView * ) obj;
                optionHandler->setScrollerActive();
                if (!(highlightWidget == sv->verticalScrollBar()))
                    sv->verticalScrollBar()->repaint(false);
                if (!(highlightWidget == sv->horizontalScrollBar()))
                    sv->horizontalScrollBar()->repaint(false);
                return FALSE;
            }
            else if ( ::qt_cast<QHeader*>(obj) ) {
                currentHeader = (QHeader*)obj;
                headerHoverID = -1;
                return FALSE;
            }
            else if (::qt_cast<QTabBar*>(obj)){ 
                currentTabBar = (QTabBar*)obj;
                hoveredTabIndex = -1;
                return FALSE;
            }
            else if ( obj->inherits( "KateView" ) || obj->inherits( "TEWidget") || obj->inherits("CHexViewWidget")) {
                QObjectList *scrollerList = obj->queryList( "QScrollBar" );
                optionHandler->setScrollerActive();
                for (uint i= 0; i < scrollerList->count(); i++)
                        ((QWidget*)(scrollerList->at(i)))->repaint(false);  
                return FALSE;                    
            }
            else if (obj->inherits("KDockWidgetAbstractHeaderDrag")){
                ((QWidget *)obj)->repaint();
                return FALSE;
            }
            else if (obj->inherits( "KAnimWidget" )){
                return TRUE;
            }
            else if ( obj->inherits( "QLineEdit" ) && obj->parent() && ::qt_cast<QComboBox*>(obj->parent())) {
                QWidget * btn = ( QWidget * ) obj->parent();
                if ( btn->isEnabled() ) {
                    highlightWidget = btn;
                    btn->repaint( false );
                }
                return FALSE;
            }
            else if ( obj->inherits( "QRadioButton" ) || obj->inherits( "QCheckBox" ) ) {
                QButton * btn = ( QButton * ) obj;
                if (btn->isEnabled()) {
                    bool isStatusChild = btn->parent() && ( btn->parent() ->inherits( "QStatusBar" ) || btn->parent() ->inherits( "KonqFrameStatusBar" ) );
                    bool isRadio = obj->inherits( "QRadioButton" );
                    highlightWidget = btn;
                    QFontMetrics fm = btn->fontMetrics();
                    QSize lsz = fm.size( ShowPrefix, btn->text() );
                    QSize sz = isRadio ?
                                QSize( pixelMetric( PM_ExclusiveIndicatorWidth ), pixelMetric( PM_ExclusiveIndicatorHeight ) ) :
                                QSize( pixelMetric( PM_IndicatorWidth ), pixelMetric( PM_IndicatorHeight ) );
                    int x = 0;
                    int y = isStatusChild ? 0 : ( btn->height() - lsz.height() + fm.height() - sz.height() ) / 2;
                    //if(btn->autoMask())
                    //    btn->erase(x+1, y+1, sz.width()-2, sz.height()-2);
                    QPainter p;
                    p.begin( btn );
                    SFlags flags = Style_Default;
                    if ( btn->isEnabled() )
                        flags |= Style_Enabled;
                    if ( btn->hasFocus() )
                        flags |= Style_HasFocus;
                    if ( btn->isDown() )
                        flags |= Style_Down;
                    if ( highlightWidget == btn )
                        flags |= Style_MouseOver;
                    if ( btn->state() == QButton::On )
                        flags |= Style_On;
                    else if ( btn->state() == QButton::Off )
                        flags |= Style_Off;
                    if ( isRadio )
                        drawControl( CE_RadioButton, &p, btn, QRect( x, y, sz.width(), sz.height() ), btn->colorGroup(), flags );
                    else
                        drawControl( CE_CheckBox, &p, btn, QRect( x, y, sz.width(), sz.height() ), btn->colorGroup(), flags );
                    p.end();
                }
                return FALSE;
            }
            return FALSE;
        case QEvent::Leave:
//             qWarning("Leave");
            if ( ::qt_cast<QToolButton*>(obj) ) {
                QToolButton * btn = ( QToolButton * ) obj;
                if (obj->inherits( "TaskContainer" )){
                    currentTaskContainer = NULL;
                    btn->repaint( false );
                    return FALSE;
                }
                else if ( btn == highlightWidget ) {
                    highlightWidget = NULL;
                    btn->repaint( false );
                    return FALSE;
                }
            }
            else if ( ::qt_cast<QPushButton*>(obj) || ::qt_cast<QComboBox*>(obj) || obj->inherits( "QSplitterHandle" ) || obj->inherits( "AppletHandleDrag" ) ) {
                QWidget * btn = ( QWidget * ) obj;
                if ( btn == highlightWidget ) {
                    highlightWidget = NULL;
                    if (::qt_cast<QPushButton*>(obj))
                        fadeOut((QPushButton*)obj);
                    else
                        btn->repaint( false );
                }
                return FALSE;
            }
            else if (::qt_cast<KToolBar*>(obj)){
                QObjectList *handlerList = obj->queryList( "QDockWindowHandle" );
                for (uint i= 0; i < handlerList->count(); i++)
                    ((QWidget*)(handlerList->at(i)))->repaint(false);
                return FALSE;
            }
//             else if (::qt_cast<QMenuBar*>(obj)){
//                 optionHandler->mouseOverMenu = false;
//                 return FALSE;
//             }
            /*else*/ if ( ::qt_cast<QScrollBar*>(obj) ) {
                QScrollBar * sb = ( QScrollBar * ) obj;
                if ( sb == highlightWidget && !sb->draggingSlider() ) {
                    activeScroller = NULL;
                    highlightWidget = NULL;
                    sb->repaint( false );
                }
                return FALSE;
            }
            else if ( ::qt_cast<QScrollView*>(obj) ) {
                QScrollView *sv = (QScrollView*)obj;
                optionHandler->setScrollerInactive();
                if (!(highlightWidget == sv->verticalScrollBar()))
                    sv->verticalScrollBar()->repaint(false);
                if (!(highlightWidget == sv->horizontalScrollBar()))
                    sv->horizontalScrollBar()->repaint(false);
                return FALSE;
            }
            else if ( ::qt_cast<QHeader*>(obj) ) {
                QHeader *hw = (QHeader*)obj;
                currentHeader = NULL;
                if ( headerHoverID != -1 ) {
                    hw->repaint( hw->sectionPos( headerHoverID )-hw->offset(), 0, hw->sectionSize( headerHoverID ), hw->height(), false );
                }
                headerHoverID = -1;
                return FALSE;
            }
            else if (::qt_cast<QTabBar*>(obj)){ 
                QTabBar *tb = (QTabBar*)obj;
                currentTabBar = NULL;
                if ( hoveredTabIndex != -1 ) {
                    QTab *tmpTab = tb->tabAt(hoveredTabIndex);
                    if (tmpTab) tb->repaint( tmpTab->rect(), false );
                }
                hoveredTabIndex = -1;
                return FALSE;
            }
            else if ( obj->inherits( "KateView" ) || obj->inherits( "TEWidget") || obj->inherits("CHexViewWidget")) {
                QObjectList *scrollerList = obj->queryList( "QScrollBar" );
                optionHandler->setScrollerInactive();
                for (uint i= 0; i < scrollerList->count(); i++)
                    ((QWidget*)(scrollerList->at(i)))->repaint(false);
                return FALSE;
            }
            else if (obj->inherits("KDockWidgetAbstractHeaderDrag")){
                ((QWidget *)obj)->repaint();
                return FALSE;
            }
            else if (obj->inherits( "KAnimWidget" )){
                return TRUE;
            }
            else if ( obj->inherits( "QLineEdit" ) && obj->parent() && ::qt_cast<QComboBox*>(obj->parent())) {
                QWidget *btn = (QWidget*)obj->parent();
                if ( btn == highlightWidget )
                    highlightWidget = NULL;
                btn->repaint( false );
                return FALSE;
            }
            else if ( obj->inherits( "QRadioButton" ) || obj->inherits( "QCheckBox" ) ) {
                QButton * btn = ( QButton * ) obj;
                if (btn == highlightWidget) {
                    bool isRadio = obj->inherits( "QRadioButton" );
                    bool isStatusChild = btn->parent() && ( btn->parent() ->inherits( "QStatusBar" ) || btn->parent() ->inherits( "KonqFrameStatusBar" ) );
                    highlightWidget = NULL;
                    QFontMetrics fm = btn->fontMetrics();
                    QSize lsz = fm.size( ShowPrefix, btn->text() );
                    QSize sz = isRadio ?
                                QSize( pixelMetric( PM_ExclusiveIndicatorWidth ), pixelMetric( PM_ExclusiveIndicatorHeight ) ) :
                                QSize( pixelMetric( PM_IndicatorWidth ), pixelMetric( PM_IndicatorHeight ) );
                    int x = 0;
                    int y = isStatusChild ? 0 : ( btn->height() - lsz.height() + fm.height() - sz.height() ) / 2;
                    //if(btn->autoMask())
                    //    btn->erase(x+1, y+1, sz.width()-2, sz.height()-2);
                    QPainter p;
                    p.begin( btn );
                    SFlags flags = Style_Default;
                    if ( btn->isEnabled() )
                        flags |= Style_Enabled;
                    if ( btn->hasFocus() )
                        flags |= Style_HasFocus;
                    if ( btn->isDown() )
                        flags |= Style_Down;
                    if ( highlightWidget == btn )
                        flags |= Style_MouseOver;
                    if ( btn->state() == QButton::On )
                        flags |= Style_On;
                    else if ( btn->state() == QButton::Off )
                        flags |= Style_Off;
                    if ( isRadio )
                        drawControl( CE_RadioButton, &p, btn, QRect( x, y, sz.width(), sz.height() ), btn->colorGroup(), flags );
                    else
                        drawControl( CE_CheckBox, &p, btn, QRect( x, y, sz.width(), sz.height() ), btn->colorGroup(), flags );
                    p.end();
                }
                return FALSE;
            }
            return FALSE;
        case QEvent::MouseButtonPress:
//             qWarning("MouseButtonPress");
            if ( ::qt_cast<QScrollBar*>(obj) ){
                QScrollBar * sb = ( QScrollBar * ) obj;
                activeScroller = NULL;
                sb->repaint( false );
                return FALSE;
            }
            else if (obj->inherits( "KAnimWidget" )){
                ((QFrame *)obj)->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
                return FALSE;
            }
//             qWarning("passing unhandled mouse press, class %s", obj->className());
            return FALSE;
        case QEvent::MouseButtonRelease:
//             qWarning("MouseButtonRelease");
            if ( ::qt_cast<QScrollBar*>(obj) ){
                QScrollBar * sb = ( QScrollBar * ) obj;
                QMouseEvent * me = ( QMouseEvent * ) ev;
                if ( sb == highlightWidget){
                    if ( sb->rect().contains( me->pos() ) ) {
                        activeScroller = sb;
                        sb->repaint( false );
                    }
                    else {
                        // this assumes, scrollers are right/bottom to scrollview
                        if ( ((sb->orientation() == Vertical) && (me->pos().x() < 0)) || ((sb->orientation() == Horizontal) && (me->pos().y() < 0)) ) {
                            optionHandler->setScrollerActive();
                            sb->repaint( false );
                        }
                    }
                }
                return FALSE;
            }
            else if ( ::qt_cast<QHeader*>(obj) && obj->parent() && ::qt_cast<QListView*>(obj->parent())) {
                QMouseEvent * me = ( QMouseEvent * ) ev;
                headerSortID = ((QHeader*)obj)->sectionAt( me->x() + ((QHeader*)obj)->offset());
                return FALSE;                
            }
            else if (obj->inherits( "KAnimWidget" )){
                ((QFrame *)obj)->setFrameShape(QFrame::NoFrame);
                return FALSE;
            }
            return FALSE;
//         case QEvent::MouseButtonDblClick:
        case QEvent::FocusIn:
//             qWarning("FocusIn");
//             if (::qt_cast<QMenuBar*>(obj)){
//                 optionHandler->mouseOverMenu = true;
//                 return FALSE;
//             }
            /*else*/ 
            if (::qt_cast<QPushButton*>(obj))
            {
                fadeIn((QPushButton*)obj);
                return FALSE;
            }
            if ( obj->inherits( "QLineEdit" ) ) {
                QObject *daddy = obj->parent();
                if ( daddy && daddy->inherits( "QSpinWidget" )){
                    ((QWidget*)daddy)->repaint(false);
                }
                else {
                    ((QWidget*)obj)->repaint(false);
                }
                return FALSE;
            }
            return FALSE;
        case QEvent::FocusOut:
//             qWarning("FocusOut");
//             if (::qt_cast<QMenuBar*>(obj)){
//                 optionHandler->mouseOverMenu = false;
//                 return FALSE;
//             }
            /*else*/
            if (::qt_cast<QPushButton*>(obj))
            {
                fadeOut((QPushButton*)obj);
                return FALSE;
            }
            if ( obj->inherits( "QLineEdit" ) ) {
                if ( obj->parent() && obj->parent()->inherits( "QSpinWidget" )){
                    ((QWidget*)obj->parent())->repaint(false);
                }
                else {
                    ((QWidget*)obj)->repaint(false);
                }
                return FALSE;
            }
            return FALSE;
        case QEvent::Move:
//             qWarning("Move");
             if ( !obj->inherits( "KActiveLabel" ))
                return FALSE;
        case QEvent::Show:
//             qWarning("Show");
            if (::qt_cast<QPushButton*>(obj))
            {
                ButtonFadeInfo *bfi_ = bfi[((QPushButton*)obj)->winId()];
                if (bfi_)
                    return FALSE;
                bfi_ = new ButtonFadeInfo();
                bfi.insert(((QPushButton*)obj)->winId(),bfi_);
                if (((QPushButton*)obj)->isDefault()) fadeIn((QPushButton*)obj);
                return FALSE;
            }
            if ( !qstrcmp(obj->name(), "kde toolbar widget") ){
                QWidget* lb = static_cast<QWidget*>(obj);
                if (lb->backgroundMode() == Qt::PaletteButton)
                    lb->setBackgroundMode(Qt::PaletteBackground);
                lb->removeEventFilter(this);
                return FALSE;
            }
            else if ( !obj->inherits( "KActiveLabel" ))
                return FALSE;
//             if (((QWidget*)obj)->backgroundOrigin() != QWidget::WindowOrigin)
//                 ((QWidget*)obj)->setBackgroundOrigin( QWidget::WindowOrigin );
//             if (::qt_cast<QListView*>(obj)){
//                 QListView *LV = (QListView*)obj;
//                 qWarning("%d",LV->selectionMode());
//                 LV->firstChild()?qWarning("%d", LV->firstChild()->height()):qWarning("NO LISTVIEW CHILD FOUND!!!");
//             }
//             if (::qt_cast<QListBox*>){
//             }
//             return FALSE;
        case QEvent::Resize:{
//             qWarning("Resize");
            QResizeEvent* rev = (QResizeEvent*) ev;
            if (::qt_cast<QPopupMenu*>(obj) && optionHandler->MenuBackground() == Gradient){
                KPixmap tmpPix = QPixmap(rev->size().width(),32);
                ((QWidget *)obj)->setPaletteBackgroundPixmap(KPixmapEffect::unbalancedGradient(tmpPix, optionHandler->Color(), optionHandler->Color2(), KPixmapEffect::HorizontalGradient, 100, 100));
            }
            else if ( obj->inherits( "KActiveLabel" )){
                QWidget * w = ( QWidget * ) obj;
                QPalette pal( w->palette() );
                QPixmap *tile = pal.brush( QPalette::Active, QColorGroup::Background ).pixmap();
                if ( tile ) {
                    QSize sz( ev->type() == QEvent::Resize ? rev->size() : w->size() );
                    QPixmap pix( sz.width(), sz.height() );
                    QPainter p;
                    if (optionHandler->BrushMe()){
                        p.begin( &pix );
                        // calc brush offsets
                        int tmpWidth;
                        if ((tmpWidth = pix.width()) > 1024){
                            int leftWidth = (tmpWidth-1024)/2;
                            int rightWidth = tmpWidth - leftWidth;
                            p.drawTiledPixmap(0, 0, leftWidth, sz.height(), plainBrush);
                            p.drawTiledPixmap(leftWidth, 0, 1024, sz.height(), gradBrush);
                            p.drawTiledPixmap(leftWidth + 1024, 0, rightWidth, sz.height(), plainBrush);
                        }
                        else{
                                p.drawTiledPixmap(0, 0, tmpWidth, sz.height(), gradBrush, (1024-tmpWidth)/2, 0);
                        }
                        p.end();
                        QBrush brush( pal.active().background(), pix );
                        pal.setBrush( QColorGroup::Base, brush );
                        w->setPalette( pal );
                    }
                    else {
//                         qWarning("y: %d, h: %d, off: %d",w->y(),w->height(),w->backgroundOffset().y());
                        p.begin(w);
                        p.drawTiledPixmap(QRect(0,0,sz.width(),sz.height()), *tile, w->backgroundOffset());
                    }
                }
                return FALSE;
            }
            else if (optionHandler->BrushMe() && obj->isWidgetType() && ((QWidget*)obj)->isTopLevel() &&
            !(::qt_cast<QPopupMenu*>(obj) || ::qt_cast<QMenuBar*>(obj) || ((QWidget*)obj)->isDesktop () || ::qt_cast<QListBox*>(obj) ||
            obj->inherits("QTipLabel") || obj->inherits("AlphaWidget") || obj->inherits("ScrollWidget") ) && (ev->type() == QEvent::Show ||
            rev->size().width() != rev->oldSize().width())){
                QWidget* widget = (QWidget*)obj;
                // create new brushpix
                QPixmap tmpBrushPix(ev->type() == QEvent::Show?widget->width():rev->size().width(), 52);
                QPainter lightMe(&tmpBrushPix);
                // calc brush offsets
                int tmpWidth;
                if ((tmpWidth = tmpBrushPix.width()) > 1024){
                    int leftWidth = (tmpWidth-1024)/2;
                    int rightWidth = tmpWidth - leftWidth;
                    lightMe.drawTiledPixmap(0, 0, leftWidth, 52, plainBrush,256-leftWidth,0);
                    lightMe.drawPixmap(leftWidth, 0, gradBrush);
                    lightMe.drawTiledPixmap(leftWidth + 1024, 0, rightWidth, 52, plainBrush);
                }
                else{
                    lightMe.drawPixmap(0, 0, gradBrush, (1024-tmpWidth)/2, 0, tmpWidth);
                }
                lightMe.end();
                    
                //update palette
                QPalette pal( qApp->palette());
                QColor c( pal.active().background() );
                QBrush brush( c, tmpBrushPix );
                pal.setBrush ( QColorGroup::Background, brush );
                //reset apps palette
                qApp->installEventFilter(eventKiller); // pssst... don't let anyone know...
                qApp->setPalette(pal);
                pal = QPalette(widget->palette());
                pal.setBrush ( QColorGroup::Background, brush );
                widget->installEventFilter(eventKiller); // pssst... don't let anyone know...
                widget->setPalette(pal);
                widget->removeEventFilter(eventKiller); // ...that we just replaced the palette
//                 qApp->lock();
                QObjectList *kids = widget->queryList("QWidget"); //this is crappy slow, but children returns a const list... f*ckf*ckf*ck
                for (uint i= 0; i < kids->count(); i++){
                    QWidget *kid = (QWidget*)kids->at(i);
                    if (kid && kid->paletteBackgroundPixmap() && !(kid->isTopLevel() || kid->inherits( "KonqIconViewWidget" ) || kid->inherits( "khtml") || (kid->parentWidget() && kid->parentWidget()->inherits("Thumbnail") || kid->parentWidget()->inherits("MarkListItem")) )){
                        //we can loose those kids anytime - especially if the user plays on konqueror...
//                         if (kid){
                            if (::qt_cast<KToolBarButton*>(kid))
                                kid->setErasePixmap(tmpBrushPix);
                            else {
                            pal = QPalette(kid->palette());
                            pal.setBrush ( QColorGroup::Background, brush );
                            kid->installEventFilter(eventKiller); // pssst... don't let anyone know...
                            kid->setPalette(pal);
                            kid->removeEventFilter(eventKiller); // ...that we just replaced the palette
//                             }
                        }
                    }
                }
                delete kids;
                qApp->removeEventFilter(eventKiller); // ...that we just replaced the palette
//                 qApp->unlock();
                return FALSE;
            }
            return FALSE;
            }
        case QEvent::Hide:
            if (::qt_cast<QPushButton*>(obj))
            {
                ButtonFadeInfo *bfi_ = bfi[((QPushButton*)obj)->winId()];
                if (bfi_)
                {
                    obj->killTimer(bfi_->timerId);
//                     qWarning("%u: stopped (%d)", ((QPushButton*)obj)->winId(), bfi_->timerId);
                    bfi_->timerId=0;
                }
            }
        case QEvent::ApplicationPaletteChange:
//             qWarning("ApplicationPaletteChange");
            if ( obj->inherits( "QTipLabel" ) ) {
                ( ( QWidget* ) obj ) ->setPalette( tooltipPalette );
                return TRUE;
            }
            return FALSE;
        case QEvent::PaletteChange:
            if (obj->inherits("QStatusBar")){
//                 qWarning("!!!!!!!!! \"%s\", %s, \"%s\"",obj->name(),obj->parent()?obj->parent()->className():"no dad",obj->parent()?obj->parent()->name():"");
                    obj->installEventFilter(eventKiller);
                    ((QWidget*)obj)->setPalette(qApp->palette());
                    obj->removeEventFilter(eventKiller);
                return TRUE;
            }
            if ( obj->inherits( "QTipLabel" ) ) {
                ( ( QWidget* ) obj ) ->setPalette( tooltipPalette );
                return TRUE;
            }
            if (obj->inherits("KCalcButton")){
                QWidget *w = (QWidget*)obj;
                QPalette pal( w->palette() );
                pal.setBrush( QColorGroup::Background, qApp->palette().brush(QPalette::Active, QColorGroup::Background));
                w->installEventFilter(eventKiller);
                w->setPalette( pal );
                w->removeEventFilter(eventKiller);
                w->setBackgroundOrigin( QWidget::WindowOrigin );
                return TRUE;
            }
            return FALSE;
        default:
            return FALSE;
    }
}

void LiquidStyle::fadeIn(QPushButton *button)
{
    ButtonFadeInfo *bfi_ = bfi[button->winId()];
    if (!bfi_)
    {
        bfi_ = new ButtonFadeInfo();
        bfi.insert(button->winId(),bfi_);
    }
    bfi_->fadeIn = TRUE;
    if (bfi_->timerId == 0)
    {
        bfi_->index = 0;
        bfi_->timerId = button->startTimer(button->isDefault()?100:50);
//         qWarning("%u: started (%d)", button->winId(), bfi_->timerId);
    }
}

void LiquidStyle::fadeOut(QPushButton *button)
{
    ButtonFadeInfo *bfi_ = bfi[button->winId()];
    if (!bfi_)
    {
        bfi_ = new ButtonFadeInfo();
        bfi.insert(button->winId(),bfi_);
    }
    bfi_->fadeIn = FALSE;
    if (bfi_->timerId == 0)
    {
        bfi_->index = 9;
        bfi_->timerId = button->startTimer(50);
//         qWarning("%u: started (%d)", button->winId(), bfi_->timerId);
    }
}

BMShower::BMShower(QObject *parent) : QObject(parent)
{
}

bool BMShower::eventFilter( QObject *obj, QEvent *ev ) {
    if (ev->type() == QEvent::Show && (obj->isWidgetType() && ((QWidget*)obj)->isTopLevel() &&
            !(::qt_cast<QPopupMenu*>(obj) || ::qt_cast<QMenuBar*>(obj) || ((QWidget*)obj)->isDesktop () || ::qt_cast<QListBox*>(obj) ||
            obj->inherits("QTipLabel") || obj->inherits("AlphaWidget") || obj->inherits("ScrollWidget") ))){
        QResizeEvent* rev = new QResizeEvent(((QWidget*)obj)->size(), QSize(0,0));
        parent()->eventFilter( obj, rev );
        obj->removeEventFilter(this);
        delete rev;
    }
    return FALSE;
}
