#include "baghira.h"
#include <qapplication.h>
#include <qbrush.h>
#include <qcolor.h>
#include <qevent.h> 
#include <qimage.h>
#include <qobject.h>
#include <qpainter.h>
#include <qpalette.h>
#include <qpopupmenu.h>
#include <qrect.h>
#include <qsettings.h>
#include <qsize.h>
#include <qstring.h>
// #include <kconfig.h>
#include <kpixmapeffect.h>
#include <stdio.h>

#define _SCALE_    65536
#define _NORMALIZE_ >>16

OptionHandler::OptionHandler( QObject *parent ) : QObject( parent ) {
	scrollerActive = false;
	groupShadowButton = false;
	reloadSettings();
	}

	
const QColor& OptionHandler::textColor() {
	return ( useCustomMenuColor ? fgColor : qApp->palette().active().text() );
	}

const QColor& OptionHandler::bgColor() {
	LiquidStyle * style = ( LiquidStyle* ) parent();
        return ( style->isKicker ? style->origPanelBrush.color() : qApp->palette().active().background() );
	}

const QColor& OptionHandler::buttonColor() {
	LiquidStyle * style = ( LiquidStyle* ) parent();
	if (style->isKicker) return style->origPanelBrush.color();
        else if (style->isOOO) return CustomButtonColor();
        else return qApp->palette().active().button();
	}

void OptionHandler::reloadSettings() {
	// gather options...
	//QSettings *settings = new QSettings;
//         KConfig *config = new KConfig("baghirarc");
//         config.setGroup("Style");
        QSettings config;
        config.beginGroup("/baghira/Style");
	//we need that first to make sure we use some proper settings ;)
	customButtonColor = QColor(config.readNumEntry("Design_ButtonColor",(int)qApp->palette().active().button().rgb()));
        // design handling:
        // first try file:
        QString tmpString;
        FILE *file = NULL;
        wmDesign = 3;
        if (qstrcmp( qApp->argv() [ 0 ], "ksplash" ) == 0)
            brushMe = FALSE;
        else
        {
        // first try for a tmp file from bab starter
        tmpString = QDir::homeDirPath() + "/.baghira/.bab/" + qApp->argv() [ 0 ];
        if( (file = fopen(tmpString.latin1(), "r")) != NULL )
            {
            int b;
            fscanf(file,"%d\n%d\n",&b, &wmDesign );
            brushMe = (b == 1);
            fclose(file);
            remove(tmpString.latin1()); // remove TEMPORARAY file
            }
        else
        {
            tmpString = QDir::homeDirPath() + "/.baghira/" + qApp->argv() [ 0 ];
            if( (file = fopen(tmpString.latin1(), "r")) != NULL )
            {
                int b;
                fscanf(file,"%d\n%d\n",&b, &wmDesign );
                brushMe = (b == 1);
                fclose(file);
            }
            else
                brushMe = config.readNumEntry( "Design_Default", 1 ) == 1;
        }
        }
        if (brushMe)
            {
            if (wmDesign > 2) wmDesign = 2;
            tintBrush = config.readBoolEntry( "Colors_TintBrushedMetal", false );
            if (tintBrush)
                brushTint.setRgb( config.readNumEntry( "Colors_BrushTint", ( int ) bgColor().rgb()));
            else
                brushedMetalColor = QColor(200, 200, 200);
            }
        bool buttonsFollow, toolButtonsFollow;
        buttonsFollow = config.readBoolEntry( "Design_ButtonsFollow", false);
        toolButtonsFollow = config.readBoolEntry( "Design_ToolButtonsFollow", true);
	// menu stuff
        menuBackground = config.readNumEntry( "Menu_Background", Standard);
	menuOpacity = config.readNumEntry( "Menu_Opacity", 70);
	menuColorButton = config.readNumEntry( "Menu_ColorStyle", 0) == 1;
	shadowText = config.readBoolEntry( "Menu_ShadowText", false);
	useCustomMenuColor = config.readNumEntry( "Menu_ColorStyle", 0) == 2;
	if (useCustomMenuColor){
		color = QColor( config.readNumEntry( "Menu_Color1", 0 ) );
                color2 = QColor( config.readNumEntry( "Menu_Color2", 0 ) );
                colorHigh = QColor( config.readNumEntry( "Menu_ColorHighlight", 0 ) );
		fgColor = QColor( config.readNumEntry( "Menu_TextColor", 0 ) );
                fgColorHigh = QColor( config.readNumEntry( "Menu_TextColorHighlight", 0 ) );
	}
        else if (menuColorButton){
            color = customButtonColor;
            color2 = customButtonColor.dark(130);
        }
        else {
            color = qApp->palette().active().background();
            color2 = qApp->palette().active().background().dark(130);
        }
        drawMenuStripe_ = config.readBoolEntry("Menu_DrawMenuStripe", false);
        if (drawMenuStripe_)
            menuStripeColor_ = QColor(config.readNumEntry("Menu_StripeColor"),(int)Qt::white.rgb());
	// color stuff
	// widgets
	customWidgetColor  =  config.readBoolEntry( "Colors_UseCustomColors", false);
	if ( customWidgetColor ) {
		customColors[ CustomRadioOn ].setRgb( config.readNumEntry( "Colors_RadioOn", ( int ) buttonColor().rgb() ) );
		customColors[ CustomRadioOff ].setRgb( config.readNumEntry( "Colors_RadioOff", ( int ) bgColor().rgb() ) );
		customColors[ CustomCBOn ].setRgb( config.readNumEntry( "Colors_CheckOn", ( int ) buttonColor().rgb() ) );
		customColors[ CustomCBOff ].setRgb( config.readNumEntry( "Colors_CheckOff", ( int ) bgColor().rgb() ) );
		customColors[ CustomTabOn ].setRgb( config.readNumEntry( "Colors_TabActive", ( int ) buttonColor().rgb() ) );
		customColors[ CustomTabOff ].setRgb( config.readNumEntry( "Colors_TabInactive", ( int ) bgColor().rgb() ) );
		customColors[ CustomSBSlider ].setRgb( config.readNumEntry( "Colors_Slider", ( int ) bgColor().rgb() ) );
		customColors[ CustomSBSliderHover ].setRgb( config.readNumEntry( "Colors_SliderHovered", ( int ) buttonColor().rgb() ) );
		customColors[ CustomSBSliderPressed ].setRgb( config.readNumEntry( "Colors_SliderPressed", ( int ) buttonColor().dark(110).rgb() ) );
		customColors[ CustomSBGroove ].setRgb( config.readNumEntry( "Colors_SliderGroove", ( int ) bgColor().rgb() ) );
	}
	// Panel
	panelCustom = config.readBoolEntry( "Special_CustomKickerColor", false );
	if (panelCustom)
		customPanelColor = QColor( config.readNumEntry( "Special_KickerColor", ( int ) bgColor().rgb() ) );
	// inactive Button
	inactiveButtonColor = config.readNumEntry( "Design_InactiveButtonStyle", Background);
	if (inactiveButtonColor == Custom)
		customInactiveButtonColor = QColor( config.readNumEntry( "Design_InactiveButtonColor", (int) bgColor().rgb()));
	//design stuff
	//stipple
	bgStipple = brushMe ? true : config.readBoolEntry( "Design_StippleBackground", true );
	panelStipple = brushMe ? true : config.readBoolEntry( "Special_StippleKicker", true );
	contrast = config.readNumEntry( "Design_StippleContrast", 3);
        bevelHighlights_ = config.readBoolEntry( "Design_BevelAsHighlight", true);
	//shadows
	groupboxshadow = brushMe ? false : config.readBoolEntry( "Design_ShadowGroupBoxes", true );
	shadowDarkness = config.readNumEntry( "Design_GroupBoxeShadowDarkness", 6);
	//ListViews
	expanderStyle = config.readNumEntry( "Special_ExpanderStyle", Apple);
	useCustomExpanderColor = config.readBoolEntry( "Special_CustomExpanderColor", false);
	if (useCustomExpanderColor)
		expanderColor = QColor( config.readNumEntry( "Special_ExpanderColor", (int) qApp->palette().active().text().rgb()));
	drawDotlines = config.readBoolEntry( "Special_DrawTreeLines", true);
	if (drawDotlines){
		dotlineStyle = config.readNumEntry( "Special_TreelineStyle", Line);
		dotlineColor = QColor( config.readNumEntry( "Special_TreelineColor", (int) qApp->palette().active().mid().rgb()));
	}
	//slider
	squeezesbslider = config.readBoolEntry( "Special_SqueezeSlider", false );
	shadowsbslider = config.readBoolEntry( "Special_ShadowSlider", false );
	animateSlider = config.readBoolEntry( "Special_AnimateSlider", true );
	// toolbar
        int utb = config.readNumEntry( "Special_UnhoveredToolButtons", 2 );
        tbFrame = utb == 2 || (brushMe && utb == 1) || (!brushMe && utb == 0);

	// tweaks
	centerTabs = config.readBoolEntry( "Design_CenterTabs", true);
	smoothListViewHeaders = !config.readBoolEntry( "Special_UseFlatLVH", false);
	smootherLVH = config.readBoolEntry( "Special_RoundTaskbuttons", false);
	icyButtons = config.readNumEntry( "Design_ButtonStyle", 0) == 0;
	progressBar = config.readNumEntry( "Special_ProgressStyle", baghira);
	removeKickerBevel = config.readBoolEntry( "Special_RemoveKickerBevel", false);
	smoothToolbuttons = toolButtonsFollow ? brushMe ? false : true : config.readBoolEntry( "Special_RoundToolButtons", true);
        forceRectButton = buttonsFollow ? brushMe ? true : false : config.readBoolEntry( "Design_RectangularButtons", false);
	drawProgressLabel = config.readBoolEntry( "Special_ShowProgressValue", false);
        tabStyle_ = (tabStyle) config.readNumEntry( brushMe?"Design_TabStyleBrushed":"Design_TabStyleAqua", Clever);
        config.endGroup();
	// Option gathered
	}
